"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RBTRowControls = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const react_bootstrap_1 = require("react-bootstrap");
const react_bootstrap_icons_1 = require("react-bootstrap-icons");
const rowcontrols_helper_1 = require("./rowcontrols.helper");
const common_1 = require("../common");
/** Unique value for filters applied by this component. */
const FILTER_VALUE = 'rowcontrols';
/**
 * This is used to create table paigniation controls at the bottom of the table
 * @param {RBTRowControlOptions} param0
 * @returns
 */
const RBTRowControls = ({ rowsPerPage, data, name, varient, handleDisplayedRows, }) => {
    const styleVarient = (0, react_1.useMemo)(() => varient ?? 'dark', [varient]);
    const tableName = (0, react_1.useMemo)(() => {
        let result = 'Row Controls';
        if (name) {
            if (name.length > 0) {
                result = name + ' ' + result;
            }
        }
        return result;
    }, [name]);
    /** How Many Table rows should be displayed? The default is 5 */
    const [paginationRows, setPaginationRows] = (0, react_1.useState)(rowsPerPage ?? 5);
    /** What is the current position of displayed rows within the table (e.g. if there are 5 rows per page and 10 records, position could be 0, 5)*/
    const [tablePosition, setTablePosition] = (0, react_1.useState)(0);
    /**
     * If a callback function has been supplied his will update the data to indicate if row controls should be applied to the data.
     *
     * @param position the position we should be centered on.
     * @param rowsPerPage  the number of rows we want to see on a 'page'
     * @param rows  the rows to be processed
     * @param rowsToMark allows us to control how much is displayed
     * @returns processed RBTRows
     */
    const updateDisplayedRows = (0, react_1.useCallback)((position, rowsPerPage, rows, rowsToMark = 0) => {
        if (typeof handleDisplayedRows === 'function') {
            const original = JSON.stringify(data);
            const updatedRows = (0, rowcontrols_helper_1.SetPaginationFilter)(position, rowsPerPage, rows, rowsToMark);
            const updated = JSON.stringify(updatedRows);
            // Use strings to check if we have added/removed rowcontrol filters to anytning
            if (original !== updated) {
                handleDisplayedRows(updatedRows);
            }
        }
    }, []);
    /**
     * This is supplied a new table position, it will iterate only data from that postion (to the range endpoint)
     * @param tablePos the new table position
     */
    const handleTablePosition = (0, react_1.useCallback)((tablePos) => {
        if (tablePosition !== tablePos) {
            setTablePosition(tablePos);
        }
        updateDisplayedRows(tablePos, paginationRows, data);
    }, [data]);
    /** Works out how many rows are visible to the user, to create a row total. */
    const numRows = (0, react_1.useMemo)(() => {
        let result = 0;
        if (data) {
            data.forEach((row) => {
                if (row) {
                    if (row.filters.length === 0) {
                        result++;
                    }
                    else {
                        // Check it isn't this component filtering the value, if it is we should include it
                        // within the count
                        const filters = (0, common_1.RemoveFilterFromArray)(FILTER_VALUE, row.filters);
                        if (filters.length === 0) {
                            result++;
                        }
                    }
                }
            });
        }
        return result;
    }, [data]);
    /** Gets the table sizes based on the number of rows of data.*/
    const rowOptions = (0, react_1.useMemo)(() => (0, rowcontrols_helper_1.getRowOptions)(numRows), [data]);
    /** Generates the text which sits next to the drop down to indicate to the user, */
    const rangeText = (0, react_1.useMemo)(() => (0, rowcontrols_helper_1.getRowRangeText)(numRows, paginationRows, tablePosition), [numRows, paginationRows, tablePosition]);
    /**
     * Called as the drop down can contain numbers and string values, so this correctly converts
     * them as appropriate.
     *
     * @param {string} value the Rows Per Page value we want to update the table to
     */
    const handleTablePagination = (value) => {
        if (value) {
            if (value === 'All') {
                setPaginationRows(numRows);
                updateDisplayedRows(0, numRows, data);
            }
            else {
                const convertedValue = Number(value);
                if (isNaN(convertedValue)) {
                    console.log('handleTablePagination: Invalid Value has been supplied ' + value);
                }
                else {
                    setPaginationRows(convertedValue);
                    updateDisplayedRows(tablePosition, convertedValue, data, convertedValue);
                }
            }
        }
    };
    (0, react_1.useEffect)(() => {
        handleTablePosition(tablePosition);
    }, [data, tablePosition]);
    return (react_1.default.createElement(react_bootstrap_1.Row, { className: 'justify-content-end mx-1 my-1' },
        react_1.default.createElement(react_bootstrap_1.Col, { xs: 'auto' },
            react_1.default.createElement("p", { className: 'pt-2' }, "Rows per page")),
        react_1.default.createElement(react_bootstrap_1.Col, { xs: 'auto' },
            react_1.default.createElement(react_bootstrap_1.Form.Select, { "aria-expanded": false, "aria-haspopup": 'listbox', "aria-label": tableName + ' Set Rows Per Table Page', className: 'mx-1 my-1', onChange: (e) => handleTablePagination(e.currentTarget.value) }, rowOptions.map((option) => {
                return (react_1.default.createElement("option", { role: 'menuitem', key: 'table.dropdown.' + option }, option));
            }))),
        react_1.default.createElement(react_bootstrap_1.Col, { xs: 'auto' },
            react_1.default.createElement("p", { className: 'pt-2' }, rangeText)),
        numRows > paginationRows ? (react_1.default.createElement(react_bootstrap_1.Col, { xs: 'auto' },
            react_1.default.createElement(react_bootstrap_1.Button, { "aria-label": tableName + ' Move to first page of table results', className: 'mx-1 my-1 px-2', onClick: () => handleTablePosition(0), disabled: tablePosition === 0, variant: styleVarient },
                react_1.default.createElement(react_bootstrap_icons_1.ChevronBarLeft, null)),
            react_1.default.createElement(react_bootstrap_1.Button, { "aria-label": tableName + ' Move to previous page of table results', className: 'mx-1 my-1 px-2', onClick: () => handleTablePosition(tablePosition - paginationRows), disabled: tablePosition - paginationRows < 0, variant: styleVarient },
                react_1.default.createElement(react_bootstrap_icons_1.ChevronLeft, null)),
            react_1.default.createElement(react_bootstrap_1.Button, { "aria-label": tableName + ' Move to next page of table results', className: 'mx-1 my-1 px-2', onClick: () => handleTablePosition(tablePosition + paginationRows), disabled: tablePosition + paginationRows >= numRows, variant: styleVarient },
                react_1.default.createElement(react_bootstrap_icons_1.ChevronRight, null)),
            react_1.default.createElement(react_bootstrap_1.Button, { "aria-label": tableName + ' Move to last page of table results', className: 'mx-1 my-1 px-2', onClick: () => handleTablePosition(numRows - paginationRows), disabled: numRows - paginationRows <= tablePosition, variant: styleVarient },
                react_1.default.createElement(react_bootstrap_icons_1.ChevronBarRight, null)))) : null));
};
exports.RBTRowControls = RBTRowControls;
//# sourceMappingURL=rowcontrols.components.js.map